//*******************************************************************************************
//
//	Definitions of CDlgPlaylist
//
// Copyright (c) 2001 JL BOYER. All rights reserved
//
//*******************************************************************************************

#ifndef _DlgPlaylist_H_
#define _DlgPlaylist_H_

#include <commctrl.h>
#include "Dlg.h"
#include "priv.h"
#include "MP3Info.h"
#include "MP3MultiInfo.h"

#define IDC_STATUS 400


class CDlgPlaylist: public CDlg
{

public:
	HWND m_hwndList;
	MP3MultiInfo m_minfo;		// store the info of all files
	CMP3Info **m_mp3iSelArray;	
	int m_nSelCount;
	tstring m_filterArtist;		// filter
	tstring m_filterAlbum;
	tstring m_filterComment;
	int m_filterMode;
	int m_filterID3Tag;
	HANDLE m_hThread;		// Thread Filter
	HANDLE m_hQuitEvent;
	BOOL m_bThreadActive;

private:
	
	int  m_realColumn[COLUMN_MAX];
	int  m_last_sort;
	BOOL m_bShowDuplicate;
	BOOL m_bRefreshSelLength;	// use :ST_UpdateLengthSelected
								//      Need to compute selected length
	long m_nTotalLength;
	long m_nAvgLength;
	long m_nSelLength;
	int  m_nFilterBy;
	HWND m_hwndStat;
	HWND m_hwndSplitter;
	HWND m_hwndTree;
	HICON m_hIcon;


	HCURSOR m_hSplitterCursor;	// Splitter variables
	BOOL m_bSplitFirstTime; 
	RECT m_SplitDragRect;
	RECT m_SplitLimitRect;
	int m_nSplitterOffset;
	BOOL m_bTracking;

	StrArray m_StrArray;	// Treeview explorer
	HIMAGELIST m_hIml;
	HTREEITEM m_hRoot[6];

	enum { COMDLG_OPEN = 0, COMDLG_SAVE, COMDLG_SAVE_AS, COMDLG_INSERT};
	enum { NO_FILTER = 0, FILTER_GENRE = 1,FILTER_ARTIST = 2,FILTER_ALBUM = 4, FILTER_COMMENT = 8, FILTER_ID3TAG = 16, FILTER_MODE = 32};
	enum { ROOT_ARTIST = 0, ROOT_GENRE, ROOT_ID3TAG, ROOT_COMMENT, ROOT_MODE, ROOT_ALBUM};
	enum eSplitterHits {   SPLITTER_HITNOTHING = 0,    SPLITTER_HITITEM, };


public:
	CDlgPlaylist() ;
	~CDlgPlaylist() ;
	void LV_ColumnGetRealOrder(int order[]);
	static void GetFrameInfo(CMP3Info *mp3i, int fid, tstring &str);
	void LV_UpdateListWithFilter();

	operator HWND() const {return(m_hDlg);}
	BOOL DestroyWindow() {BOOL bRet=::DestroyWindow(m_hDlg); m_hDlg = NULL; return(bRet);}

protected:
	virtual BOOL RealDlgProc(UINT uMsg, WPARAM wParam, LPARAM lParam);

private:
	void Init();
	void LoadParam(void);
	void SaveParam(void);
	LRESULT NotifyListView(LPARAM lParam);
	LRESULT NotifyTreeView(LPARAM lParam);
	int InsertFile(tstring& file);
	static int CALLBACK ListViewCompareProc(LPARAM lParam1, LPARAM lParam2, LPARAM lParamSort); //int nSubItem);
	BOOL doContextMenu(WPARAM wParam, LPARAM lParam);

	void LV_ColumnResetDisplay(BOOL firstime);
	void LV_ColumnSort(int column, BOOL bColumn, BOOL bReverse=TRUE);
	int  LV_ColumnGetNum();
	void LV_ColumnSaveSetting();
	void LV_InitIcon();
	void LV_ColumnsGrid();
	void LV_DropFiles(HANDLE hFilesInfo);
	void LV_AddFiles();
	void LV_AddDirectory();
	void LV_AddDir2List(TCHAR* dir);
	void LV_New();
	void LV_ComDlg(int OPENSAVE);
	void LV_Import();
	void LV_Export();
	void AddFileExtension(TCHAR *szFile, TCHAR *szExt); 
	void LV_Playlist(int);
	void LV_PlaylistWinamp(int);
	void LV_LoadEZ3(TCHAR *filename, BOOL bInitList=TRUE);
	void LV_LoadM3U(TCHAR *filename);
	void LV_LoadPLS(TCHAR *filename);
	void LV_WriteEZ3(TCHAR*);
	BOOL LV_WriteM3U(TCHAR*);
	BOOL LV_WritePLS(TCHAR*);
	BOOL LV_WriteHTML_Table(TCHAR*);
	BOOL LV_WriteHTML_Listing(TCHAR*);
	void LV_WriteHTML_TableColumn(FILE *file, CMP3Info *mp3i, int iCol, int iColArtist);
	void LV_WriteHTML_CSS(FILE *file, TCHAR *title);
	void LV_WriteHTML_Footer(FILE *file, TCHAR *szCol, unsigned long uTotalSize, long nTotalLength, int nCountSelect);
	BOOL LV_WriteCSV(TCHAR*);
	BOOL LV_WriteASCII(TCHAR*);
	BOOL LV_WriteASCIIFixed(TCHAR* filename);
	void LV_SelectAll(BOOL bSelect);
	void LV_SelectToggle();
	void LV_SelectWithSame(int nCol);
	void LV_SelectCrop();
	void LV_SelectDel();
	void LV_SelectRemoveFile();
	void LV_UpdateStatus();
	void LV_StatGenre();
	void LV_UpdatePlaylist();
	void LV_DetermineFilter();
	BOOL LV_Filter(CMP3Info *mp3i);
	void LV_TitleWindowFilter();
	void LV_ShowDuplicate();
	void LV_DeadFiles();
	void LV_Redraw(BOOL bUpdate);
	void LV_GetInfoCol(CMP3Info *mp3i, int col, TCHAR *szCol, int iItem = -1);
	void LV_MatchTime(int nLengthMax);

	void UpdateMenu(HMENU hMenu); 
	void MenuButton(int nButton);				
	void MenuCheckItem(int nItem);				
	void OwnerDrawBtn (LPDRAWITEMSTRUCT lpdis);
	void SetActive();

	void NoSelection();
	void GetMP3Selected();
	void SetMP3Selected();
	
	void ST_UpdateLengthSelected();
	BOOL ST_CreateStatus(int nMode);
	BOOL ST_ChangeText(LPTSTR szText, int iPart );

	void winamp_enqueue_to_playlist(BOOL bPlay);
	void winamp_enqueue_all_to_playlist(BOOL bPlay);

	void UpdateStatus();

	void OnSize(UINT nState, int nWidth, int nHeight);
	void ResizeControls();

	BOOL TV_Create();
	void TV_Destroy();
	void TV_Load();
	void TV_Update();
	void TV_Redraw(BOOL bUpdate);
	void TV_SetCurrentItem();

	int AddVector(tstring& str);
	int FindVector(tstring& str);


	void Splitter_Init(void);
	void Splitter_OnInvertTracker(HWND hWnd, const RECT *rect);
	void Splitter_Recalc(void);
	void Splitter_StartTracking(HWND hWnd, UINT hitArea);
	void Splitter_StopTracking(HWND hWnd);
	void Splitter_OnMouseMove(HWND hWnd, UINT nFlags, POINTS p);
	void Splitter_OnLButtonDown(HWND hWnd, UINT nFlags, POINTS p);
	void Splitter_OnLButtonUp(HWND hWnd, UINT nFlags, POINTS p);
	UINT Splitter_HitTest(HWND hWnd, POINTS p);	

	BOOL LaunchThreadFilter();
	void SetTextMenu();
	void SetTextMenuPop(HMENU menu, int nType);
	void RefreshAllCtrl_Lang();
	void SetNameColumn();
};

#endif